(function (factory) {
    'use strict';
    if (typeof define === 'function' && define.amd) {
        // AMD. Register as an anonymous module.
        define(['jquery', 'jquery-elementresize'], factory);
    } else if (typeof exports === 'object') {
        // Node/CommonJS style for Browserify/Webpack
        module.exports = factory(require('jquery', 'jquery-elementresize'));
    } else {
        // Browser globals
        factory(jQuery);
    }
}(function ($) {    
    'use strict';
    
    var pluginName = 'scrollButtons',
        unpluginName = 'unscrollButtons',
        
        defaults = { 
            onscroll: undefined, 
            btnTemplate: '<div class="sb-scrollbtn"></div>' 
        };

    function Plugin(elem, options) {
        this.elem = elem;
        this.$elem = $(elem);
        this.settings = $.extend(true, {}, defaults, options);
        this.init();
    }

    $.extend(Plugin.prototype, {
        init: function () {

            var cssPosition = this.$elem.css('position'),
                cssOverflow = this.$elem.css('overflow');
            
            if (cssPosition === 'static') {
                throw 'Unsupported position value: ' + cssPosition;
            } else if (cssOverflow !== 'hidden') {
                throw 'Unsupported overflow value: ' + cssOverflow;
            }

            // create the left/right buttons:            
            this.$leftButton = $(this.settings.btnTemplate)
                .addClass('sb-scrollbtn-left')
                .appendTo(this.$elem);
            this.$leftButton.data('original-display', this.$leftButton.css('display'))
                .on('click.scrollbuttons', function(width) {return width;}, $.proxy(this.button_clickHandler, this));
            this.$rightButton = $(this.settings.btnTemplate)
                .addClass('sb-scrollbtn-right')
                .appendTo(this.$elem);            
            this.$rightButton.data('original-display', this.$rightButton.css('display'))
                .on('click.scrollbuttons', function(width) {return -width;}, $.proxy(this.button_clickHandler, this));
            // get the scrollable content:           
            this.$scrollable = this.$elem.find('.sb-scrollable');            
            this.$scrollable.data('original-display', this.$scrollable.css('display'))                               
                .wrapAll('<div class="sb-scrollWrapper"></div>');            
            this.$scrollableWrapper = this.$elem.find('.sb-scrollWrapper');
            // add resize event handlers:
            this.$scrollableWrapper.on('elementResize.scrollbuttons', $.proxy(this.elementResizeHandler, this));
            this.$elem.on('elementResize.scrollbuttons',$.proxy(this.elementResizeHandler, this));            
            // initial synchronization:
            this.update();
        },
        
        
        destroy: function () {
            this.$leftButton.off('.scrollbuttons');
            this.$rightButton.off('.scrollbuttons');
            this.$scrollableWrapper.off('.scrollbuttons');
            this.$elem.off('.scrollbuttons');            
            this.$leftButton.remove();
            this.$rightButton.remove();
            this.$scrollable.unwrap();
            this.$scrollable.css('display',this.$scrollable.data('original-display'));
            
        },
        
        update: function () {
            var margin = parseInt(this.$scrollableWrapper.css('margin-left').replace('px', '')),
                totalWidth = this.$scrollableWrapper.width(),
                viewableWidth = this.$elem.width(),
                maxSlicesCount = (totalWidth / viewableWidth) - 1,
                maxMarginOffset = maxSlicesCount * viewableWidth,
                newMargin = 0;

            if (viewableWidth >= totalWidth) {
                newMargin = 0;
            } else {               
                newMargin = margin;
                if (margin > 0) {
                    newMargin = 0;
                } else {
                    var currentSlicesWidth = -margin,
                        currentSlicesCount = currentSlicesWidth / viewableWidth;
                    if (currentSlicesCount > maxSlicesCount) {                        
                        newMargin = -maxMarginOffset;
                    }
                }
            }

            // set the change:
            if (newMargin !== margin) {
                this.$scrollableWrapper.css('margin-left', newMargin + 'px');
                margin = newMargin;
            }

            // show/hide buttons:
            var hideBoth = viewableWidth >= totalWidth,
                hideLeft = hideBoth || margin === 0,
                hideRight = hideBoth || margin === -maxMarginOffset;

            this.$leftButton.css('display', hideLeft ? 'none' : this.$leftButton.data('original-display') );
            this.$rightButton.css('display', hideRight ? 'none' : this.$rightButton.data('original-display'));
        },
        
        elementResizeHandler: function () {
            this.update();
        },
        
        button_clickHandler: function (event) {
            var width = this.$elem.width(),
                margin = parseInt(this.$scrollableWrapper.css('margin-left').replace('px', '')),
                newMargin = margin + event.data(width);
            this.$scrollableWrapper.css('margin-left', newMargin + 'px');
            this.update();
        },
    });
        
    $.fn[pluginName] = function (options) {
        this.each(function () {
            if (!$.data(this, pluginName)) {
                $.data(this, pluginName, new Plugin(this, options));
            }
        });
        return this;
    };
    
    $.fn[unpluginName] = function () {
        this.each(function () {
            var plugin = $.data(this, pluginName);
            if (plugin) {
                plugin.destroy();
                $.data(this, pluginName, null);
            }
        });
        return this;
    };
    
}));
